.include "2313def.inc"

;*****************************************************************************************
;	Variablen
;*****************************************************************************************
.def	Digit			= r15	; Digit to change
.def	DelayTemp		= r16	; Temporary Variable for Delay Routine
.def	Temp			= r17	; Temporary Variable
.def	Temp2			= r18	; Temporary Variable
.def	StatusProg		= r19	; Menu Configuration
					; Bit0	1	Count forward
					; Bit1	1	Count 1 time
					; Bits 32	00	normal count mode
					;			01	set digit to change
					;			10	change digit
.def	Lcd1			= r20
.def	Lcd2			= r21
.def	Lcd3			= r22
.def	Lcd4			= r23
.def	Lcd5			= r24
.def	Lcd6			= r25
.def	Lcd7			= r26
.def	Lcd8			= r27
.def	PortTemp		= r28


;*****************************************************************************************
;	Werte
;*****************************************************************************************
.equ	LCD_PORT     	= PORTB	
.equ	LCD_DDR	     	= DDRB
.equ	LCD_PIN	     	= PINB
.equ	LCD_CRTL     	= PORTD
.equ	LCD_CRTL_DDR 	= DDRD
.equ	LCD_CRTL_PIN 	= PIND
.equ	LCD_CRTL_E	 	= 6
.equ	LCD_CRTL_RS	 	= 5
.equ	Trans8Bit		= 0b00000011
.equ	Trans4Bit		= 0b00000010
.equ	ClearDisp		= 0b00000001
.equ	Start1Line		= 0b00000010
.equ	Start2Line		= 0b11000000
			;	Set the Display
				;              +------ Data:  0 = 4-bit; 1 = 8-bit
				;              |+----- Lines: 0 = 1; 1 = 2
				;              ||+---- Font:  0 = 5x8; 1 = 5x11
.equ	SetDisp			= 0b00101100
		; ***  'CURSOR/DISPLAY SHIFT' command
				;               +----- S/C:  0 = cursor; 1 = display
				;		        |+---- R/L:  0 = left; 1 = right
.equ	ShCursRight		= 0b00010100
		; ***  'DISPLAY ON/OFF' command
				;                +---- Display: 0 = off; 1 = on
				;                |+--- Cursor: 0 = off; 1 = on
				;                ||+-- Blink: 0 = off; 1 = on
.equ	DispOn			= 0b00001100
.equ	CursSmallBlink	= 0b00001101
.equ	CursSmallOn		= 0b00001110
		; *** 'ENTRY MODE' command
			;					  +--- Direction: 0 = left; 1 = right
			;					  |+-- Shift Dislay: 0 = off; 1 = on
.equ	ShiftCursRight	= 0b00000110
.equ	CountDir		= 0		; Count direction
.equ	Countit			= 1		; Count 1 time

;*****************************************************************************************
;	Programmcode
;*****************************************************************************************
.cseg
.org	$0000

;*****************************************************************************************
;	Interruptroutinen
;*****************************************************************************************
		rjmp	START			; Reset Vektor
		rjmp	INT0_ROUTINE	; Int0 Pin
		rjmp	INT1_ROUTINE	; Int1 Pin
		reti					; Timer1 Capture Event
		reti					; Timer1 Compare Event
		reti					; Timer1 Overflow
		reti					; Timer0 Overflow
		reti					; UART RX Complete
		reti					; UART Data Register Empty
		reti					; UART TX Complete
		reti					; Analog Comparator

;*****************************************************************************************
;	Daten fr das Display
;*****************************************************************************************
Clean:		.db "        "
Welcome:	.db " Zaehler"
			.db " V0.910 "

;*****************************************************************************************
;	Programmstart
;*****************************************************************************************
START:
		ldi		Temp, low(RAMEND)		; Initialize Stackpointer
    	out   	SPL, Temp
		cli								; Disable all Interrupts
		clr		Temp
		out		GIMSK, Temp				; Disable Interrupts Int0 and Int1
		out		TIMSK, Temp				; Disable all Timer Interrupts
		ldi		Temp, 0b01100000		; Set LCD Control Pins as output
		out		LCD_CRTL_DDR, Temp
		rcall	DELAY4ms				; Time for the Display to start up
		rcall	DISP_TRANS				; Set the Transfer mode
		ldi		Temp, ClearDisp			; Clear the Display
		rcall	LCD_SEND_CMD
		rcall	DELAY4ms
		ldi		Temp, SetDisp			; Set Up the Display Lines and Fonts
		rcall	LCD_SEND_CMD
		ldi		Temp, ShCursRight		; Set Display/Cursor Shift and Direction
		rcall	LCD_SEND_CMD
		ldi		Temp, DispOn			; Set Display/Cursor ON/OFF
		rcall	LCD_SEND_CMD
		ldi		Temp, ShiftCursRight	; Set the Cursor/Display Right/left Shift during Entry
		rcall	LCD_SEND_CMD
		ldi		Temp, Start1Line		; Send Cursor to Beginning of first line
		rcall	LCD_SEND_CMD
		rcall	DELAY4ms
		ldi		ZL, low(Welcome*2)		; Set Pointer to Menu Text
		ldi		ZH, high(Welcome*2)
		ldi		Temp2, 8
		rcall	LCD_PRINT_PM			; Print the Text
		ldi		Temp, Start2Line		; Send Cursor to beginning of second Line
		rcall	LCD_SEND_CMD
		ldi		Temp2, 8
		rcall	LCD_PRINT_PM			; Print the next Line
		rcall	DELAY1s					; Wait 1s
		rcall	DELAY1s					; Wait 1s
		rcall	DELAY1s					; Wait 1s
		rcall	DELAY1s					; Wait 1s
		rcall	DELAY1s					; Wait 1s
		clr		StatusProg
		rcall	NUMBER_CLEAR
		rcall	SET_COUNTER_FORWARD
		ldi		Temp, 0b00001010		; Set Interrupt events to falling edges of int0 and int1
		out		MCUCR, Temp
		sei								; enable global interrupts

forever:
		sbrc	StatusProg, Countit
		rcall	NUMBER_TO_LCD
		in		Temp, PINB
		andi	Temp, 0b00001111		; only take the used inputs
		cp		Temp, Porttemp
		in		Temp2, SREG
		sbrs	Temp2, 1
		rcall	SWITCH_PRESSED			; call Switch control routine if switch changed
		rjmp	forever					; end program here

;*****************************************************************************************
;	Interruptroutinen
;*****************************************************************************************

INT0_ROUTINE:
		sbis	PIND, 3
		rcall	COUNT_UP
		sbic	PIND, 3
		rcall	COUNT_DOWN
		ori		StatusProg, 0b00000010
		reti
INT1_ROUTINE:
		sbis	PIND, 2
		rcall	COUNT_UP
		sbic	PIND, 2
		rcall	COUNT_DOWN
		ori		StatusProg, 0b00000010
		reti

;*****************************************************************************************
;	Subroutinen
;*****************************************************************************************

SWITCH_PRESSED:
		mov		Temp2, Temp				; make backup
		eor		Temp2, Porttemp			; find the changed Bits
		mov		Porttemp, Temp			; update Porttemp Register
		sbrc	Temp2, 0
		rjmp	SWITCH_0_CHANGED		; Bit 0 changed
		sbrc	Temp2, 1
		rjmp	SWITCH_1_CHANGED		; Bit 1 changed
		sbrc	Temp2, 2
		rcall	SWITCH_2_CHANGED		; Bit 2 changed
		sbrc	Temp2, 3
		rjmp	SWITCH_3_CHANGED		; Bit 3 changed
		rjmp	SWITCH_PRESSED_OUT
 SWITCH_0_CHANGED:						; + switch
		sbrc	Temp, 0
		rjmp	SWITCH_PRESSED_OUT		; go out if switch 0 is not pressed
		mov		Temp2, StatusProg
		andi	Temp2, 0b00001100
		cpi		Temp2, 0b00000000
		in		Temp, SREG
		sbrc	Temp, 1
		rcall	CHANGE_COUNT_DIR		; switch 0 pressed, clear Numbers
		cpi		Temp2, 0b00000100
		in		Temp, SREG
		sbrc	Temp, 1
		rcall	DIGIT_DOWN
		cpi		Temp2, 0b00001000
		in		Temp, SREG
		sbrc	Temp, 1
		rcall	VALUE_UP
		rjmp	SWITCH_PRESSED_OUT		; go out
 SWITCH_1_CHANGED:						; edit switch
		sbrc	Temp, 1
		rjmp	SWITCH_PRESSED_OUT		; go out if switch 1 is not pressed
		mov		Temp2, StatusProg
		andi	Temp2, 0b00001100
		cpi		Temp2, 0b00000000
		in		Temp, SREG
		sbrc	Temp, 1
		rcall	EDIT_DIGITS
		cpi		Temp2, 0b00000100
		in		Temp, SREG
		sbrc	Temp, 1
		rcall	EDIT_OUT
		cpi		Temp2, 0b00001000
		in		Temp, SREG
		sbrc	Temp, 1
		rcall	SET_VALUE_OUT
		rjmp	SWITCH_PRESSED_OUT		; go out
 SWITCH_2_CHANGED:						; set switch
		sbrc	Temp, 2
		rjmp	SWITCH_PRESSED_OUT		; go out if switch 2 is not pressed
		mov		Temp2, StatusProg
		andi	Temp2, 0b00001100
		cpi		Temp2, 0b00000100
		in		Temp, SREG
		sbrc	Temp, 1
		rcall	SET_DIGIT
		cpi		Temp2, 0b00001000
		in		Temp, SREG
		sbrc	Temp, 1
		rcall	SET_VALUE_OUT
		rjmp	SWITCH_PRESSED_OUT		; go out
 SWITCH_3_CHANGED:						; - switch
		sbrc	Temp, 3					; switch 3 is not pressed, go out
		rjmp	SWITCH_PRESSED_OUT
		mov		Temp2, StatusProg
		andi	Temp2, 0b00001100
		cpi		Temp2, 0b00000000
		in		Temp, SREG
		sbrc	Temp, 1
		rcall	NUMBER_CLEAR
		cpi		Temp2, 0b00000100
		in		Temp, SREG
		sbrc	Temp, 1
		rcall	DIGIT_UP
		cpi		Temp2, 0b00001000
		in		Temp, SREG
		sbrc	Temp, 1
		rcall	VALUE_DOWN
		rjmp	SWITCH_PRESSED_OUT		; go out
 SWITCH_PRESSED_OUT:
		ret
DIGIT_UP:						; increases the Digit variable by 1
								;		and sets the cursor at it
		push	Temp
		mov		Temp, Digit				; load Digit into Temp
		cpi		Temp, 7
		breq	DIGIT_UP_LEAST_DIGIT	; compare if already at highest Digit
		inc		Temp					; increase Temp
		rjmp	DIGIT_UP_OUT
 DIGIT_UP_LEAST_DIGIT:
		ldi		Temp, 1					; load least Digit valueinto Temp
 DIGIT_UP_OUT:
		mov		Digit, Temp				; store Digit
		rcall	SET_CURSOR_DIGIT
		pop		Temp
		ret

DIGIT_DOWN:						; decreases the Digit variable by 1
								;		and sets the cursor at it
		push	Temp
		mov		Temp, Digit				; load Digit into Temp
		cpi		Temp, 2
		brlo	DIGIT_DOWN_HIGHEST_DIGIT; compare if already at least Digit
		dec		Temp
		rjmp	DIGIT_DOWN_OUT
 DIGIT_DOWN_HIGHEST_DIGIT:
		ldi		Temp, 7					; load highest Digit valueinto Temp
 DIGIT_DOWN_OUT:
		mov		Digit, Temp				; store Digit
		rcall	SET_CURSOR_DIGIT
		pop		Temp
		ret

SET_CURSOR_DIGIT:				; move the cursor to the digit determined by Digit
		push	Temp2
		mov		Temp2, Digit
		ldi		Temp, Start1Line		; go to beginning of first line
		rcall	LCD_SEND_CMD
 SET_CURSOR_DIGIT_SHIFT:				; execute this loop Digit times
		ldi		Temp, ShCursRight		; shift the cursor right
		rcall	LCD_SEND_CMD
		dec		Temp2
		brne	SET_CURSOR_DIGIT_SHIFT
		pop		Temp2
		ret

VALUE_DOWN:						; decreases the actual Digit (selected with Digit)
								;		by 1 and sends its contents to the display
								;		and sets the cursor at the digit again
		push	ZH
		push	ZL
		push	Temp
		clr		ZH
		ldi		ZL, 20
		add		ZL, Digit
		ld		Temp, Z
		cpi		Temp, 0x30
		breq	VALUE_DOWN_SET_MAX
		dec		Temp
		rjmp	VALUE_DOWN_OUT
 VALUE_DOWN_SET_MAX:
		ldi		Temp, 0x39
 VALUE_DOWN_OUT:
		st		Z, Temp
		rcall	NUMBER_TO_LCD
		pop		Temp
		pop		ZL
		pop		ZH
		ret

VALUE_UP:						; increases the actual Digit (selected with Digit)
								;		by 1 and sends its contents to the display
								;		and sets the cursor at the digit again
		push	ZH
		push	ZL
		push	Temp
		clr		ZH
		ldi		ZL, 20
		add		ZL, Digit
		ld		Temp, Z
		cpi		Temp, 0x39
		breq	VALUE_UP_SET_MIN
		inc		Temp
		rjmp	VALUE_UP_OUT
 VALUE_UP_SET_MIN:
		ldi		Temp, 0x30
 VALUE_UP_OUT:
		st		Z, Temp
		rcall	NUMBER_TO_LCD
		pop		Temp
		pop		ZL
		pop		ZH
		ret

EDIT_DIGITS:					; sets the Digit variable to 1, sets the StatusProg
								; 		Edit Mode Bits to 01, moves the cursor to
								; 		Digit 1 and makes it a small
								;		 visible one
		push	Temp
		ldi		Temp, 7
		mov		Digit, Temp					; set Digit to 8
		andi	StatusProg, 0b11110111		; Set status bits 32 to 01
		ori		StatusProg, 0b00000100
		ldi		Temp, CursSmallOn			; make the cursor a small one
		rcall	LCD_SEND_CMD
		rcall	SET_CURSOR_DIGIT			; set the cursoe to digit 1
		pop		Temp
		ret

EDIT_OUT:						; set StatusProg edit mode bits to 00 and let the
								; 		cursor vanish
		push	Temp
		andi	StatusProg, 0b11110011
		ldi		Temp, DispOn
		rcall	LCD_SEND_CMD
		pop		Temp
		ret

SET_VALUE_OUT:					; Sets Statusprog edit mode bits to 01 and change
								;		the cursor into the small one
		push	Temp
		andi	StatusProg, 0b11110111
		ori		StatusProg, 0b00000100
		ldi		Temp, CursSmallOn
		rcall	LCD_SEND_CMD
		rcall	SET_CURSOR_DIGIT
		pop		Temp
		ret

SET_DIGIT:						; set StatusProg edit mode bits to 10 and change the
								; 		cursor into the small blinking one
		push	Temp
		andi	StatusProg, 0b11111011
		ori		StatusProg, 0b00001000
		ldi		Temp, CursSmallBlink
		rcall	LCD_SEND_CMD
		rcall	SET_CURSOR_DIGIT
		pop		Temp
		ret

CHANGE_COUNT_DIR:				; switches StatusProg,0, changes the contents of Lcd1
								; 		and switches the ext. Interrupt
		sbrc	StatusProg, 0
		rjmp	SET_COUNTER_FORWARD
		rjmp	SET_COUNTER_BACKWARD
		pop		Temp
		ret

SET_COUNTER_FORWARD:			; set Counter Direction forward by Int0-activation
								;		and load corresponding sign into Lcd1
		push	Temp
		andi	StatusProg, 0b11111110	; Reset Direction Bit
		ori		StatusProg, 0b00000010	; Set send Lcd Bit
		ldi		Lcd1, 0b00111110		; load forward sign to Lcd1
		ldi		Temp, 0b01000000		; set Int0-Pin
		out		GIMSK, Temp 
		pop		Temp
		ret

SET_COUNTER_BACKWARD:			; set Counter Direction backward by Int1-activation
								;		and load corresponding sign into Lcd1
		push	Temp
		ori		StatusProg, 0b00000001	; set Direction Bit
		ori		StatusProg, 0b00000010	; Set send Lcd Bit
		ldi		Lcd1, 0b00111100		; load backward sign to Lcd1
		ldi		Temp, 0b10000000		; set Int1-Pin
		out		GIMSK, Temp
		pop		Temp
		ret

COUNT_UP:						; increases the Display number 1 time
		inc		Lcd8
		cpi		Lcd8, 0x3A
		brlo	COUNT_UP_OUT
		ldi		Lcd8, 0x30
		inc		Lcd7
		cpi		Lcd7, 0x3A
		brlo	COUNT_UP_OUT
		ldi		Lcd7, 0x30
		inc		Lcd6
		cpi		Lcd6, 0x3A
		brlo	COUNT_UP_OUT
		ldi		Lcd6, 0x30
		inc		Lcd5
		cpi		Lcd5, 0x3A
		brlo	COUNT_UP_OUT
		ldi		Lcd5, 0x30
		inc		Lcd4
		cpi		Lcd4, 0x3A
		brlo	COUNT_UP_OUT
		ldi		Lcd4, 0x30
		inc		Lcd3
		cpi		Lcd3, 0x3A
		brlo	COUNT_UP_OUT
		ldi		Lcd3, 0x30
		inc		Lcd2
		cpi		Lcd2, 0x3A
		brlo	COUNT_UP_OUT
		ldi		Lcd2, 0x30
 COUNT_UP_OUT:
		ret

COUNT_DOWN:
		dec		Lcd8
		cpi		Lcd8, 0x30
		brsh	COUNT_DOWN_OUT
		ldi		Lcd8, 0x39
		dec		Lcd7
		cpi		Lcd7, 0x30
		brsh	COUNT_DOWN_OUT
		ldi		Lcd7, 0x39
		dec		Lcd6
		cpi		Lcd6, 0x30
		brsh	COUNT_DOWN_OUT
		ldi		Lcd6, 0x39
		dec		Lcd5
		cpi		Lcd5, 0x30
		brsh	COUNT_DOWN_OUT
		ldi		Lcd5, 0x39
		dec		Lcd4
		cpi		Lcd4, 0x30
		brsh	COUNT_DOWN_OUT
		ldi		Lcd4, 0x39
		dec		Lcd3
		cpi		Lcd3, 0x30
		brsh	COUNT_DOWN_OUT
		ldi		Lcd3, 0x39
		dec		Lcd2
		cpi		Lcd2, 0x30
		brsh	COUNT_DOWN_OUT
		ldi		Lcd2, 0x39
 COUNT_DOWN_OUT:		
		ret


NUMBER_CLEAR:					; sets all Display digits to zero
		ldi		Lcd2, 0x30
		ldi		Lcd3, 0x30
		ldi		Lcd4, 0x30
		ldi		Lcd5, 0x30
		ldi		Lcd6, 0x30
		ldi		Lcd7, 0x30
		ldi		Lcd8, 0x30
		ori		StatusProg, 0b00000010
		ret

NUMBER_FULL:					; sets all Display digits to LCD 9
		ldi		Lcd2, 0x39
		ldi		Lcd3, 0x39
		ldi		Lcd4, 0x39
		ldi		Lcd5, 0x39
		ldi		Lcd6, 0x39
		ldi		Lcd7, 0x39
		ldi		Lcd8, 0x39
		ret

NUMBER_TO_LCD:					; transmit Data of Lcd1 to Lcd8 to the display
		push	Temp
		ldi		Temp, Start1Line		; Send Cursor to Beginning of first line
		rcall	LCD_SEND_CMD
		rcall	DELAY4ms				; *** Always use this Delay between a command and the Text entry!!
		pop		Temp
		ldi		ZH,0
		ldi		ZL, 20					; Set Pointer to Lcd1
		ldi		Temp2, 8
		rcall	LCD_PRINT_MEM
		mov		Temp, StatusProg
		cpi		Temp, 0b00000000
		breq	NUMBER_TO_LCD_OUT
		rcall	SET_CURSOR_DIGIT
 NUMBER_TO_LCD_OUT:
		andi	StatusProg, 0b11111101
		ret

LCD_PRINT_PM:						; *** LCD_PrintPM: Prints from program memory
									; Put the starting memory location in Z (r31:r30)
									; Put the number of characters to print in Temp2
									; After execution, Z is at the character AFTER
									; the last to be printed and Temp2 is zero.
									; This function will not wrap if you the string
									; is bigger than the LCD.
		push	r0
		push	Temp
 LCD_PM_READ: 
		lpm
    	mov		Temp, r0
		rcall	LCD_SEND_CHAR
		adiw	r30, 1
		dec		Temp2
		brne	LCD_PM_READ
		pop		Temp
		pop		r0
		ret

LCD_PRINT_MEM:						; *** LCD_PrintMem: Prints from memory.
									; Put the starting memory location in Z (r31:r30)
									; Put the number of characters to print in Temp2
									; After execution, Z is at the character AFTER
									; the last to be printed and Temp2 is zero.
									; This function will not wrap if you the string
									; is bigger than the LCD.
		push	Temp
 LCD_MEM_READ: 
   		ld		Temp, Z+
		rcall	LCD_SEND_CHAR
		dec		Temp2
		brne	LCD_MEM_READ
		pop		Temp
		ret

LCD_SEND_CHAR:						; *** Send the Contents of Temp to the LCD using 4bit Mode
		push 	Temp
		push 	Temp2
		rcall	DISP_OUT
		mov 	Temp2, Temp			; Make a backup copy
		andi 	Temp2, 0b11110000	; Only use the upper nibble
		swap	Temp2				; move it to lower outputs
		out		LCD_PORT, Temp2		; Send it
		rcall	LCD_RS_HIGH
		rcall 	LCD_PULSEE			; Pulse the enable
		rcall	DELAY775
		andi	Temp, 0b00001111	;use lower nibble of Temp
		out		LCD_PORT, Temp		; Send it
		rcall 	LCD_PULSEE			; Pulse the enable
		rcall	DELAY775
		rcall	LCD_RS_LOW			; Reset strobe signal
		rcall	DISP_IN
		pop 	Temp2
		pop 	Temp
		ret

DISP_TRANS:							; ************** Set the Transfer mode
		push	Temp
		rcall	DISP_OUT
		ldi		Temp, Trans8Bit		; set the Display in 8bit mode first
		out		LCD_PORT, Temp
		rcall	LCD_RS_LOW
		rcall	LCD_PULSEE			; Send
		cbi		LCD_PORT,0			; Now it's safe to go into 4-bit mode.
		rcall	LCD_PULSEE
		rcall 	DELAY775
		rcall	DISP_IN
		pop		Temp
		ret

LCD_SEND_CMD:						; ************** LCD_SendCmd: Routine to write a command
									; to the instruction register.
									; The value to be written should be stored in Temp.  
									; The value is sent 4 bits at a time.
		push 	Temp
		push	Temp2
		mov 	Temp2, Temp			; Make a backup copy
		rcall	DISP_OUT
		andi 	Temp2, 0b11110000	; Only use the upper nibble
		swap	Temp2				; move it to the lower ports
		out		LCD_PORT, Temp2		; Send it
		rcall 	LCD_PULSEE			; Pulse the enable
		andi 	Temp, 0b00001111	; Only use the lower nibble
		out		LCD_PORT, Temp		; Send it
		rcall 	LCD_PULSEE			; Pulse the enable
		rcall	DELAY775
		rcall	DISP_IN
		pop		Temp2
		pop 	Temp
		ret

LCD_RS_HIGH:	
		sbi		LCD_CRTL, LCD_CRTL_RS
		ret

LCD_RS_LOW:
		cbi		LCD_CRTL, LCD_CRTL_RS
		ret		

LCD_PULSEE:	
		sbi		LCD_CRTL, LCD_CRTL_E
		nop
		nop
		cbi		LCD_CRTL, LCD_CRTL_E
		ret

DISP_OUT:							; *** Set the LCD Data Pins as Output
		push	Temp
		ldi		Temp, 0b00001111
		out		LCD_DDR, Temp
		pop		Temp
		ret

DISP_IN:							; *** Set the LCD Data Pins as Input
		push	Temp
		ldi		Temp, 0b00000000
		out		LCD_DDR, Temp
		out		LCD_PORT, Temp
		pop		Temp
		ret

DELAY1s:							; *** provide 3.971.125 MZ delay
		push	DelayTemp
		ldi		DelayTemp, 255
 DELAYCOUNT1s:
		rcall	DELAY4ms
		dec		DelayTemp
		brne	DELAYCOUNT1s
		pop		DelayTemp
		ret

DELAY4ms:
		push	DelayTemp
		ldi		DelayTemp, 20
 DELAYCOUNT4ms:
 		rcall	DELAY775
 		dec		DelayTemp
		brne	DELAYCOUNT4ms
		pop		DelayTemp
		ret

DELAY775:					; *** provide 775 MZ delay including the rcall command
		push	DelayTemp				; 2MZ
		ldi		DelayTemp, 255			; 1MZ
 DELAYCOUNT:
 		dec		DelayTemp				; 1MZ
		brne	DELAYCOUNT				; 1/2MZ
		pop		DelayTemp				; 2MZ
		ret								; 3MZ, rcall 3MZ